@@ -13,6 +13,7 @@
 #    under the License.
 
 import collections
+import errno
 import multiprocessing
 import os
 import signal
@@ -23,9 +24,7 @@ import threading
 import time
 from unittest import mock
 
-from oslo_config import cfg
 from oslotest import base as test_base
-import six
 
 from oslo_concurrency.fixture import lockutils as fixtures
 from oslo_concurrency import lockutils
@@ -186,6 +185,13 @@ class LockTestCase(test_base.BaseTestCase):
                 pass
         self.assertRaises(NotImplementedError, do_test)
 
+    def test_fair_lock_with_nonblocking(self):
+        def do_test():
+            with lockutils.lock('testlock', 'test-', fair=True,
+                                blocking=False):
+                pass
+        self.assertRaises(NotImplementedError, do_test)
+
     def test_nested_synchronized_external_works(self):
         """We can nest external syncs."""
         self.config(lock_path=tempfile.mkdtemp(), group='oslo_concurrency')
@@ -278,10 +284,7 @@ class LockTestCase(test_base.BaseTestCase):
         # Note(flaper87): Lock is not external, which means
         # a semaphore will be yielded
         with lockutils.lock("test") as sem:
-            if six.PY2:
-                self.assertIsInstance(sem, threading._Semaphore)
-            else:
-                self.assertIsInstance(sem, threading.Semaphore)
+            self.assertIsInstance(sem, threading.Semaphore)
 
             # NOTE(flaper87): Lock is external so an InterProcessLock
             # will be yielded.
@@ -295,10 +298,7 @@ class LockTestCase(test_base.BaseTestCase):
         self.config(lock_path=tempfile.mkdtemp(), group='oslo_concurrency')
 
         with lockutils.lock("test") as sem:
-            if six.PY2:
-                self.assertIsInstance(sem, threading._Semaphore)
-            else:
-                self.assertIsInstance(sem, threading.Semaphore)
+            self.assertIsInstance(sem, threading.Semaphore)
 
             with lockutils.lock("test2", external=True) as lock:
                 self.assertTrue(lock.exists())
@@ -327,8 +327,8 @@ class LockTestCase(test_base.BaseTestCase):
         remove_mock.assert_called_once_with(path_mock.return_value)
         log_mock.assert_not_called()
 
-    @mock.patch('logging.Logger.info')
-    @mock.patch('os.remove', side_effect=OSError)
+    @mock.patch('logging.Logger.warning')
+    @mock.patch('os.remove', side_effect=OSError(errno.ENOENT, None))
     @mock.patch('oslo_concurrency.lockutils._get_lock_path')
     def test_remove_lock_external_file_doesnt_exists(self, path_mock,
                                                      remove_mock, log_mock):
@@ -339,6 +339,20 @@ class LockTestCase(test_base.BaseTestCase):
                                           mock.sentinel.prefix,
                                           mock.sentinel.lock_path)
         remove_mock.assert_called_once_with(path_mock.return_value)
+        log_mock.assert_not_called()
+
+    @mock.patch('logging.Logger.warning')
+    @mock.patch('os.remove', side_effect=OSError(errno.EPERM, None))
+    @mock.patch('oslo_concurrency.lockutils._get_lock_path')
+    def test_remove_lock_external_file_permission_error(
+            self, path_mock, remove_mock, log_mock):
+        lockutils.remove_external_lock_file(mock.sentinel.name,
+                                            mock.sentinel.prefix,
+                                            mock.sentinel.lock_path)
+        path_mock.assert_called_once_with(mock.sentinel.name,
+                                          mock.sentinel.prefix,
+                                          mock.sentinel.lock_path)
+        remove_mock.assert_called_once_with(path_mock.return_value)
         log_mock.assert_called()
 
     def test_no_slash_in_b64(self):
@@ -346,18 +360,6 @@ class LockTestCase(test_base.BaseTestCase):
         with lockutils.lock("foobar"):
             pass
 
-    def test_deprecated_names(self):
-        paths = self.create_tempfiles([['fake.conf', '\n'.join([
-            '[DEFAULT]',
-            'lock_path=foo',
-            'disable_process_locking=True'])
-        ]])
-        conf = cfg.ConfigOpts()
-        conf(['--config-file', paths[0]])
-        conf.register_opts(lockutils._opts, 'oslo_concurrency')
-        self.assertEqual('foo', conf.oslo_concurrency.lock_path)
-        self.assertTrue(conf.oslo_concurrency.disable_process_locking)
-
 
 class FileBasedLockingTestCase(test_base.BaseTestCase):
     def setUp(self):
@@ -422,6 +424,35 @@ class FileBasedLockingTestCase(test_base.BaseTestCase):
                 time.sleep(.5)
                 os._exit(0)
 
+    def test_interprocess_nonblocking_external_lock(self):
+        """Check that we're not actually blocking between processes."""
+
+        nb_calls = multiprocessing.Value('i', 0)
+
+        @lockutils.synchronized('foo', blocking=False, external=True,
+                                lock_path=self.lock_dir)
+        def foo(param):
+            """Simulate a long-running operation in a process."""
+            param.value += 1
+            time.sleep(.5)
+
+        def other(param):
+            foo(param)
+
+        process = multiprocessing.Process(target=other, args=(nb_calls, ))
+        process.start()
+        # Make sure the other process grabs the lock
+        start = time.time()
+        while not os.path.exists(os.path.join(self.lock_dir, 'foo')):
+            if time.time() - start > 5:
+                self.fail('Timed out waiting for process to grab lock')
+            time.sleep(0)
+        process1 = multiprocessing.Process(target=other, args=(nb_calls, ))
+        process1.start()
+        process1.join()
+        process.join()
+        self.assertEqual(1, nb_calls.value)
+
     def test_interthread_external_lock(self):
         call_list = []
 
@@ -457,6 +488,62 @@ class FileBasedLockingTestCase(test_base.BaseTestCase):
         thread.join()
         self.assertEqual(['other', 'other', 'main', 'main'], call_list)
 
+    def test_interthread_nonblocking_external_lock(self):
+        call_list = []
+
+        @lockutils.synchronized('foo', external=True, blocking=False,
+                                lock_path=self.lock_dir)
+        def foo(param):
+            """Simulate a long-running threaded operation."""
+            call_list.append(param)
+            time.sleep(.5)
+            call_list.append(param)
+
+        def other(param):
+            foo(param)
+
+        thread = threading.Thread(target=other, args=('other',))
+        thread.start()
+        # Make sure the other thread grabs the lock
+        start = time.time()
+        while not os.path.exists(os.path.join(self.lock_dir, 'foo')):
+            if time.time() - start > 5:
+                self.fail('Timed out waiting for thread to grab lock')
+            time.sleep(0)
+        thread1 = threading.Thread(target=other, args=('main',))
+        thread1.start()
+        thread1.join()
+        thread.join()
+        self.assertEqual(['other', 'other'], call_list)
+
+    def test_interthread_nonblocking_internal_lock(self):
+        call_list = []
+
+        @lockutils.synchronized('foo', blocking=False,
+                                lock_path=self.lock_dir)
+        def foo(param):
+            # Simulate a long-running threaded operation.
+            call_list.append(param)
+            time.sleep(.5)
+            call_list.append(param)
+
+        def other(param):
+            foo(param)
+
+        thread = threading.Thread(target=other, args=('other',))
+        thread.start()
+        # Make sure the other thread grabs the lock
+        start = time.time()
+        while not call_list:
+            if time.time() - start > 5:
+                self.fail('Timed out waiting for thread to grab lock')
+            time.sleep(0)
+        thread1 = threading.Thread(target=other, args=('main',))
+        thread1.start()
+        thread1.join()
+        thread.join()
+        self.assertEqual(['other', 'other'], call_list)
+
     def test_non_destructive(self):
         lock_file = os.path.join(self.lock_dir, 'not-destroyed')
         with open(lock_file, 'w') as f:
